@@ -29,6 +29,7 @@ import yaml
 from pymongo import MongoClient
 from sets import Set
 from ferry.install import *
+from ferry.docker.resolve       import DefaultResolver
 from ferry.docker.docker        import DockerInstance
 from ferry.docker.fabric        import DockerFabric
 from ferry.docker.configfactory import ConfigFactory
@@ -41,6 +42,7 @@ class DockerManager(object):
         # Generate configuration.
         self.docker = DockerFabric()
         self.config = ConfigFactory()
+        self.resolver = DefaultResolver()
 
         # Service mappings
         self.service = {
@@ -582,23 +584,23 @@ class DockerManager(object):
 
         return container_dir, log_dir, host_name, ports, exposed
 
-    def _read_key_dir(self):
+    def _read_key_dir(self, private_key):
         """
         Read the directory containing the key we should use. 
         """
-        # This needs to be fixed. When starting the
-        # container, we need to know which key is being used. 
-        # However, we want to read the client directory instead of
-        # the server directory!
-        with open(ferry.install._get_key_dir(root=False, server=False), 'r') as f:
-            k = f.read().strip().split("://")
-            return { '/service/keys' : k[1]  }
+        return { '/service/keys' : os.path.dirname(private_key)  }
+
+    def _read_public_key(self, private_key):
+        s = private_key.split("/")
+        p = os.path.splitext(s[len(s) - 1])[0]
+        return p
 
     def _prepare_storage_environment(self, 
                                      service_uuid, 
                                      num_instances, 
                                      storage_type, 
                                      layers,
+                                     key_name, 
                                      args = None,
                                      replace = False):
         """
@@ -633,7 +635,9 @@ class DockerManager(object):
                               'type':t, 
                               'volumes':dir_info,
                               'volume_user':DEFAULT_FERRY_OWNER, 
-                              'keys': self._read_key_dir(), 
+                              'keydir': self._read_key_dir(key_name), 
+                              'keyname': self._read_public_key(key_name), 
+                              'privatekey': key_name, 
                               'ports':ports,
                               'exposed':exposed, 
                               'hostname':host_name,
@@ -647,6 +651,7 @@ class DockerManager(object):
                                      service_uuid, 
                                      num_instances, 
                                      compute_type,
+                                     key_name, 
                                      layers, 
                                      args = None):
         """
@@ -673,7 +678,9 @@ class DockerManager(object):
             container_info = {'image':instance_type,
                               'volumes':dir_info,
                               'volume_user':DEFAULT_FERRY_OWNER, 
-                              'keys': self._read_key_dir(), 
+                              'keydir': self._read_key_dir(key_name), 
+                              'keyname': self._read_public_key(key_name), 
+                              'privatekey': key_name, 
                               'type':t, 
                               'ports':ports,
                               'exposed':exposed, 
@@ -696,6 +703,7 @@ class DockerManager(object):
     def _prepare_connector_environment(self, 
                                        service_uuid, 
                                        connector_type, 
+                                       key_name, 
                                        instance_type=None,
                                        name=None, 
                                        ports=[],
@@ -714,7 +722,9 @@ class DockerManager(object):
         else:
             host_name = name
         container_info = { 'image':instance_type,
-                           'keys': self._read_key_dir(), 
+                           'keydir': self._read_key_dir(key_name), 
+                           'keyname': self._read_public_key(key_name), 
+                           'privatekey': key_name, 
                            'volumes':{},
                            'type':connector_type, 
                            'ports':ports,
@@ -737,7 +747,7 @@ class DockerManager(object):
             logging.warning("transfer config %s -> %s" % (from_dir, to_dir))
             self.docker.copy([container], from_dir, to_dir)
 
-    def _transfer_ip(self, ips):
+    def _transfer_ip(self, private_key, ips):
         """
         Transfer the hostname/IP addresses to all the containers. 
         """
@@ -745,8 +755,8 @@ class DockerManager(object):
             for ip in ips:
                 hosts_file.write("%s %s\n" % (ip[0], ip[1]))
         for ip in ips:
-            self.docker.copy_raw(ip[0], '/tmp/instances', '/service/sconf/instances')
-            self.docker.cmd_raw(ip[0], '/service/sbin/startnode hosts')
+            self.docker.copy_raw(private_key, ip[0], '/tmp/instances', '/service/sconf/instances')
+            self.docker.cmd_raw(private_key, ip[0], '/service/sbin/startnode hosts')
         
     def _transfer_env_vars(self, containers, env_vars):
         """
@@ -792,7 +802,7 @@ class DockerManager(object):
                 for c in s['containers']:
                     self.docker.stop([c])
 
-    def register_stack(self, backends, connectors, base, uuid=None):
+    def register_stack(self, backends, connectors, base, key=None, uuid=None):
         """
         Register the set of services under a single cluster identifier. 
         """
@@ -813,6 +823,7 @@ class DockerManager(object):
                     'ts':ts }
 
         if not uuid:
+            cluster['key'] = key
             self.cluster_collection.insert( cluster )
         else:
             self._update_stack(uuid, cluster)
@@ -953,6 +964,7 @@ class DockerManager(object):
 
     def allocate_compute(self,
                          compute_type, 
+                         key_name, 
                          storage_uuid,
                          args, 
                          num_instances=1,
@@ -966,7 +978,12 @@ class DockerManager(object):
 
         # Generate the data volumes. This basically defines which
         # directories on the host get mounted in the container. 
-        plan = self._prepare_compute_environment(service_uuid, num_instances, compute_type, layers, args)
+        plan = self._prepare_compute_environment(service_uuid = service_uuid, 
+                                                 num_instances = num_instances, 
+                                                 compute_type = compute_type, 
+                                                 key_name = key_name,
+                                                 layers = layers, 
+                                                 args = args)
 
         # Get the entry point for the storage layer. 
         storage_entry = self._get_service_configuration(storage_uuid)
@@ -983,14 +1000,12 @@ class DockerManager(object):
 
         # Now copy over the configuration.
         self._transfer_config(config_dirs)
-        for k in self._read_key_dir().keys():
-            keydir = k
 
+        # Update the service configuration. 
         container_info = self._serialize_containers(containers)
         service = {'uuid':service_uuid, 
                    'containers':container_info, 
                    'class':'compute',
-                   'keys': keydir, 
                    'type':compute_type,
                    'entry':entry_point,
                    'storage':storage_uuid, 
@@ -1066,6 +1081,7 @@ class DockerManager(object):
                         
     def allocate_storage(self, 
                          storage_type, 
+                         key_name, 
                          num_instances=1,
                          layers=[], 
                          args=None,
@@ -1080,7 +1096,13 @@ class DockerManager(object):
 
         # Generate the data volumes. This basically defines which
         # directories on the host get mounted in the container. 
-        plan = self._prepare_storage_environment(service_uuid, num_instances, storage_type, layers, args, replace)
+        plan = self._prepare_storage_environment(service_uuid = service_uuid, 
+                                                 num_instances = num_instances, 
+                                                 storage_type = storage_type, 
+                                                 layers = layers, 
+                                                 key_name = key_name, 
+                                                 args = args, 
+                                                 replace = replace)
 
         # Allocate all the containers. 
         containers = self._start_containers(plan)
@@ -1152,6 +1174,7 @@ class DockerManager(object):
 
     def manage_stack(self,
                      stack_uuid,
+                     private_key, 
                      action):
         """
         Manage the stack.
@@ -1189,6 +1212,7 @@ class DockerManager(object):
         """
         cluster = self.cluster_collection.find_one( {'uuid':uuid} )
         if cluster:
+            key = cluster['key']
             backends = []
             for i, uuid in enumerate(cluster['backends']['uuids']):
                 storage_uuid = uuid['storage']
@@ -1203,7 +1227,9 @@ class DockerManager(object):
                 
                 backends.append( {'storage' : storage_conf,
                                   'compute' : compute_confs} )
-            return backends
+            return backends, key
+        else:
+            return None, None
             
 
     def fetch_snapshot_backend(self, snapshot_uuid):
@@ -1253,6 +1279,7 @@ class DockerManager(object):
 
     def allocate_deployed_connectors(self, 
                                      app_uuid, 
+                                     key_name, 
                                      backend_info,
                                      conf = None):
         """
@@ -1266,6 +1293,7 @@ class DockerManager(object):
         if app:
             for c in app['connectors']:
                 uuid, containers = self.allocate_connector(connector_type = c['type'],
+                                                           key_name = key_name,  
                                                            backend = backend_info,
                                                            name = c['name'], 
                                                            args = c['args'],
@@ -1280,6 +1308,7 @@ class DockerManager(object):
                 
     def allocate_snapshot_connectors(self, 
                                      snapshot_uuid, 
+                                     key_name, 
                                      backend_info):
         """
         Lookup the snapshot connector info and instantiate. 
@@ -1290,6 +1319,7 @@ class DockerManager(object):
         if snapshot:
             for s in snapshot['snapshot_cs']:
                 uuid, containers = self.allocate_connector(connector_type = s['type'],
+                                                           key_name = key_name, 
                                                            backend = backend_info,
                                                            name = s['name'], 
                                                            args = s['args'],
@@ -1352,6 +1382,7 @@ class DockerManager(object):
                 
     def allocate_connector(self,
                            connector_type, 
+                           key_name, 
                            backend=None,
                            name=None, 
                            args=None,
@@ -1380,6 +1411,7 @@ class DockerManager(object):
         service_uuid = self._new_service_uuid()
         plan = self._prepare_connector_environment(service_uuid = service_uuid, 
                                                    connector_type = connector_type, 
+                                                   key_name = key_name, 
                                                    instance_type = image,
                                                    name = name,
                                                    ports = ports, 
